/*
** pork_proto.h
** Copyright (C) 2003 Ryan McCabe <ryan@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#ifndef __PORK_PROTO_H
#define __PORK_PROTO_H

struct buddy;
struct pork_acct;
struct imwindow;
struct file_transfer;

enum {
	PROTO_NULL = -1,
	PROTO_AIM,
	PROTO_MAX,
};

struct pork_proto {
	int protocol;
	char name[32];

	int (*connect)(struct pork_acct *);
	int (*buddy_block)(struct pork_acct *, char *);
	int (*buddy_permit)(struct pork_acct *, char *);
	int (*buddy_unblock)(struct pork_acct *, char *);
	int (*buddy_remove_permit)(struct pork_acct *, char *);
	int (*buddy_search)(struct pork_acct *, char *str);
	int (*buddy_add)(struct pork_acct *, struct buddy *);
	int (*buddy_alias)(struct pork_acct *, struct buddy *);
	int (*buddy_remove)(struct pork_acct *, struct buddy *);
	int (*buddy_update)(struct pork_acct *, struct buddy *, void *);
	int (*chat_ignore)(struct pork_acct *, char *, char *);
	int (*chat_unignore)(struct pork_acct *, char *, char *);
	int (*chat_join)(struct pork_acct *, char *);
	int (*chat_leave)(struct pork_acct *, char *);
	int (*chat_invite)(struct pork_acct *, char *, char *, char *);
	int (*chat_name)(char *name, char *buf, size_t len);
	int (*chat_send)(struct pork_acct *, char *, char *);
	int (*chat_who)(struct pork_acct *, char *);

	int (*file_send)(struct file_transfer *xfer);
	int (*file_accept)(struct file_transfer *xfer);
	int (*file_abort)(struct file_transfer *xfer);
	int (*file_recv_data)(struct file_transfer *xfer, char *buf, size_t len);
	int (*file_send_data)(struct file_transfer *xfer, char *buf, size_t len);
	int (*file_recv_complete)(struct file_transfer *xfer);
	int (*file_send_complete)(struct file_transfer *xfer);

	int (*set_report_idle)(struct pork_acct *, int);
	int (*set_privacy_mode)(struct pork_acct *, int);
	int (*set_idle_time)(struct pork_acct *, u_int32_t);
	int (*warn)(struct pork_acct *, char *);
	int (*warn_anon)(struct pork_acct *, char *);
	int (*send_msg)(struct pork_acct *, char *target, char *msg);
	int (*send_msg_auto)(struct pork_acct *, char *target, char *msg);
	int (*set_away_msg)(struct pork_acct *, char *);
	int (*get_away_msg)(struct pork_acct *, char *);
	int (*set_profile)(struct pork_acct *, char *);
	int (*get_profile)(struct pork_acct *, char *);
	int (*init)(struct pork_acct *);
	int (*free)(struct pork_acct *);
	int (*read_config)(struct pork_acct *);
	int (*write_config)(struct pork_acct *);
	int (*keepalive)(struct pork_acct *);
	int (*update)(struct pork_acct *);
	int (*normalize)(const char *str, char *buf, size_t len);
};

struct chatroom {
	char *title;
	void *data;
	u_int32_t num_users;
	dlist_t *user_list;
	struct imwindow *win;
};

int proto_init(u_int32_t protocol, int (*init_func)(struct pork_proto *));
void proto_destroy(void);
inline struct pork_proto *proto_get(u_int32_t protocol);

#endif
